/*===============================================================================
Title: dwLaunchPadWifi.ino
Description: deviceWISE TI LaunchPad with WiFi Sample Program

Copyright (c)2014, 2015 ILS Technology.
http://www.ilstechnology.com
http://www.devicewise.com


Legal Disclaimer:

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Neither the name of ILS Technology nor Telit nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

===================================================================================*/

//#define WIFI_AVAILABLE 1 

#ifdef WIFI_AVAILABLE

  #include <SPI.h>
  #include <WiFi.h>
  #include <WifiIPStack.h>
  
  //Wi-Fi Network - SSID / Password
  #define WIFI_SSID          "User_SSID"
  #define WIFI_PW            "User_PW"
  
#else

  #include <Ethernet.h>
  #include <EthernetClient.h>
  #include <EthernetServer.h>
  #include <EthernetUdp.h>
  
#endif

#include <dwOpenClient.h>


// Authentication/Registration Details  		
#define DWOPEN_APPTOKEN    "1kagVL7HGh6JAd9w"      //Application Token
#define DWOPEN_SERVER      "api.devicewise.com"    //deviceWISE server
#define DWOPEN_PORT        1883                    //MQTT Port

//ILST Headquarters - GPS Detail...
#define ILST_Latitude      "26.394919"
#define ILST_Longitude     "-80.112808"
#define ILST_Altitude      "3.0"
#define ILST_Speed         "0.0"
#define ILST_Heading       "0.0"

char dwThingKey[40];
byte mac[6] = {0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5};

void callback(char* topic, uint8_t* payload, unsigned int length);

#ifdef WIFI_AVAILABLE
  WiFiClient c;
#else  
  EthernetClient c;
#endif


//#define GROVE_BASE 1                        //Leave commented out if not using Grove Booster Pack

#ifdef GROVE_BASE
  #include <DHT.h>                          //Grove Base Include
  
  #define ANGLE_PIN         24              //J6 - Pin#24 - Angle/Rotary Sensor  
  #define TEMP_HUMI_PIN     27              //J9 - Pin#27 - Temp/Humidity Sendor
  
  DHT dht(TEMP_HUMI_PIN, DHT22);            //Grove Base temperature&humidity sensor object
#endif

dwOpenClient dwClient((char*) DWOPEN_SERVER, DWOPEN_PORT, callback, c);


void setup() 
{
  int i;
  
  Serial.begin(9600);
  while( !Serial ) delay( 100 );  
  
  for(i = 10; i > 0; i-- )
  {
    Serial.print("Ready to Execute... Countdown = ");
    Serial.println(i); 
    delay( 1000 );
  }
 
#ifdef WIFI_AVAILABLE
  Serial.print("Attempting to connect to Network named: "); Serial.println(WIFI_SSID); 
  WiFi.begin(WIFI_SSID, WIFI_PW);
  
  while( WiFi.status() != WL_CONNECTED ) 
  {
    Serial.print(".");
    delay(5000);
    WiFi.begin(WIFI_SSID, WIFI_PW);
    Serial.print("WiFi Status = "); Serial.println(WiFi.status());
  }
  
  Serial.println("\nYou're connected to the network\n\rWaiting for an ip address");
  
  while (WiFi.localIP() == INADDR_NONE) 
  {
    Serial.print(".");
    delay(300);
  }

  Serial.print("\nIP Address obtained : "); Serial.println(WiFi.localIP());
  
  //Get my MAC address
  WiFi.macAddress(mac);
  
#else

  // start the Ethernet connection:
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    // no point in carrying on, so do nothing forevermore:
    for(;;)
      ;
  }
  // give the Ethernet shield a second to initialize:
  delay(1000);
  Serial.println("connecting...");

  // if you get a connection, report back via serial:
  if (c.connect(DWOPEN_SERVER, DWOPEN_PORT)) {
    Serial.println("connected");
  } 
  else {
    // kf you didn't get a connection to the server:
    Serial.println("connection failed");
  }
#endif

#ifdef GROVE_BASE 
     dht.begin();                         /* initialize temperature humidity sensor */ 
#endif

  //Format the ThingKey for this run using our MAC
  sprintf(dwThingKey, "LP-%x%x%x%x%x%x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);  
  Serial.print("My Thing Key: ");  Serial.println( dwThingKey );   
  
  //Configure GREEN & RED LEDs
  pinMode(GREEN_LED, OUTPUT);
  pinMode(RED_LED, OUTPUT);
  
}

void loop() 
{
  int rc = -1;
  char szText[128];
  char szPropValue[16];

  if( !dwClient.connected() )
  {
      Serial.print("Connecting to: "); Serial.print(DWOPEN_SERVER); Serial.print(" with client id: "); Serial.println(dwThingKey);
  
      dwClient.connect((char*) dwThingKey, (char*) dwThingKey, (char*) DWOPEN_APPTOKEN);
  
      Serial.println("Connected to m2mAIR Cloud...");
       
      //Serial.println("Subscribing to Topic 'reply'...");
      dwClient.subscribe(MQTT_REPLY);
      
      //Serial.println("Subscribing to Topic 'notify'...");
      dwClient.subscribe(MQTT_NOTIFY);
      
      // Publish Log Message... Connected
      Serial.println("dwLaunchPad- Log Publish In Process..." );
      rc = dwClient.dwLogPublish( (char*) dwThingKey, (char*) "dwLaunchPad Demo Program Started...");
       
      Serial.print("Publish Log Message return code : "); Serial.println(rc);   
      
      // Publish Location Message...
      Serial.println("dwOpen API - Location Publish In Process..." );      
      rc = dwClient.dwLocationPublish( dwThingKey, ILST_Latitude, ILST_Longitude, ILST_Altitude, ILST_Speed, ILST_Heading );    
      
      Serial.print("Publish Location return code : "); Serial.println(rc);          
      
      // Publish Attribute...
      Serial.println("dwOpen API - Attribute Publish In Process..." );    
      sprintf(szPropValue, "%x%x%x%x%x%x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);  	  
      rc = dwClient.dwAttributePublish( dwThingKey, "MAC", szPropValue );    
      
      Serial.print("Publish Location return code : "); Serial.println(rc);         
  }
 
   if( dwClient.connected() )   
   {          

#ifdef GROVE_BASE     

      uint32_t valADC = (uint32_t) analogRead(ANGLE_PIN);  // read the value from the sensor
      Serial.print(dwThingKey);      
      Serial.print(", RotarySensor:");      
      Serial.println(valADC); 
      sprintf(szPropValue,"%d", valADC);
      
      dwClient.dwPropertyPublish( dwThingKey, "RotarySensor", szPropValue );      
      
      int Temperature = dht.readTemperature();             /* read the temperature value from the sensor */
      
      Serial.print(dwThingKey);      
      Serial.print(", Temperature:");      
      Serial.println(Temperature); 
      sprintf(szPropValue,"%d", Temperature);
      
      dwClient.dwPropertyPublish( dwThingKey, "Temperature", szPropValue );         
      
      int Humidity = dht.readHumidity();                   /* read the humidity value from the sensor */    
 
      Serial.print(dwThingKey);      
      Serial.print(", Humidity:");      
      Serial.println(Humidity); 
      sprintf(szPropValue,"%d", Humidity);
      
      dwClient.dwPropertyPublish( dwThingKey, "Humidity", szPropValue ); 
      
#else
  #ifdef TEMPSENSOR
        uint32_t valADC = (uint32_t) analogRead(TEMPSENSOR);  //Internal Temp ADC value
        Serial.print(dwThingKey);      
        Serial.print(", ADC:");      
        Serial.println(valADC); 
        sprintf(szPropValue,"%d", valADC);
        
        dwClient.dwPropertyPublish( dwThingKey, "ADC", szPropValue );  
  #else
        static uint32_t valPulse;
       
        (valPulse % 10) ? valPulse-- : valPulse++;
      
        Serial.print(dwThingKey);      
        Serial.print(", valPulse:");      
        Serial.println(valPulse); 
        sprintf(szPropValue,"%d", valPulse);
        
        dwClient.dwPropertyPublish( dwThingKey, "Pulse", szPropValue );  
  #endif      
#endif

          
  }
  else
  {
      Serial.println("Error, Connecting to m2mAIR Cloud...");
  }

  Serial.println("Sleeping for 5 Seconds..." );  
  dwClient.dwSleep(5000);
  
}


void ExtractKeyValue( char *haystack, char *Key, char *ResultValue )
{
  char *KeyPtr;
  char *KeyValue;
  char *KeyValueEnd;
  int   KeyValueLen;
	
  char needleTmp[40];
    	
    strcpy( needleTmp, "\"");
    strcat( needleTmp, Key);
    strcat( needleTmp, "\":\"");

    KeyPtr = strstr(haystack, needleTmp );
    	
    if( KeyPtr != NULL) 
    {	  
        KeyValue = KeyPtr + strlen(needleTmp);
        KeyValueEnd = KeyValue;
		
	while( *KeyValueEnd != '\"')
	{
	    KeyValueEnd++;
	}
		
        KeyValueLen = KeyValueEnd - KeyValue;
	strncpy( ResultValue, KeyValue, KeyValueLen );
	*(ResultValue + KeyValueLen ) = NULL;
    }
    else
    {
	strcpy( ResultValue, "" );
    }

}
        
// Sets the LED accordingly
void setLED( int LEDPin, char *LEDStatus )
{
    if ( strcmp(LEDStatus, "1") == NULL )
    {        
        Serial.println("LED On!!!");          
        digitalWrite(LEDPin, HIGH);
    }
    else
    {
        Serial.println("LED Off!!!");          
        digitalWrite(LEDPin, LOW);          
    }  
}

//TR50 JSON Sample Request/Response
#define MAILBOX_IN_PROGRESS_ACK  "{\"999-999\": { \"command\": \"mailbox.check\", \"params\": { \"autoComplete\": true, \"limit\": 0 }}}"
#define MAILBOX_REQ_RECEIVED     "{\"0000123\": { \"command\": \"mailbox.update\", \"params\": {\"id\": \"%s\", \"msg\": \"Executing...\"}}}"
#define MAILBOX_REQ_ACK          "{\"0000124\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"%s\", \"errorCode\": \"0\", \"errorMessage\": \"OK\"}}}"

// The function is called when an MQTT message arrives
void callback(char *topic, uint8_t *payload, unsigned int length) 
{
  int i;
  char jsonText[1024];
  char LEDStatus[10];   
  char msgID[50];

  //Serial.println("Callback - MQTT Message Arrived...");
  //Serial.print("Topic: ");  Serial.println( topic ); 
  
  //Serial.print("Payload Length: ");  Serial.println( length ); 
  //Serial.print("Payload: ");  Serial.println( (char *) payload );
  
  if(strstr(topic, MQTT_NOTIFY) != NULL) 
  {  
      //Serial.println("Notification - Mailbox Activity... Check Mailbox");
      dwClient.publish("api", MAILBOX_IN_PROGRESS_ACK );    
  }
  else if( (strstr(topic, MQTT_REPLY) != NULL) )
  {
      if ((strstr((char *)payload, "999-999") != NULL)) 
      {     
        //Serial.println("Reply - Mailbox Check Response... Mailbox Request Received");   
        ExtractKeyValue( (char *) payload, "id", msgID ); 
        
        sprintf( jsonText, MAILBOX_REQ_RECEIVED, msgID );  
  	dwClient.publish("api", jsonText );
  
        //Set LED accordingly                 
        ExtractKeyValue( (char *) payload, "greenled", LEDStatus );                
        if ( strlen(LEDStatus) ) setLED(GREEN_LED, LEDStatus);
  
        ExtractKeyValue( (char *) payload, "redled", LEDStatus );                
        if ( strlen(LEDStatus) ) setLED(RED_LED, LEDStatus);
          
    	//Serial.println("Processing Complete - Send Mailbox Acknowledge...");
        sprintf( jsonText, MAILBOX_REQ_ACK, msgID );   
    	dwClient.publish( "api", jsonText );  
      } 
    
  }
  
}



